from django.db import models


class ProjectPanelModel(models.Model):
    KB, MB, GB, TB = 'kb', 'mb', 'gb', 'tb'
    UNIT_SIZE = {KB: 1, MB: 2, GB: 3, TB: 4}

    name = models.CharField(max_length=255, verbose_name='')

    class Meta:
        managed = True
        db_table = 'ProjectPanelModel'
        verbose_name = 'Панель управления проектом'
        default_permissions = ()
        permissions = (
            ('view_panel', 'Панель управления проектом - просмотр'),
            ('manage_panel', 'Панель управления проектом - управление'),
        )

    def __str__(self):
        return f"{self.id} {self.name}"






from django.conf import settings
from django.apps import apps
from django.db import connections

import psutil, os, copy


class ProjectPanel:
    KB, MB, GB, TB = 'kb', 'mb', 'gb', 'tb'
    UNIT_SIZE = {KB: 1, MB: 2, GB: 3, TB: 4}
    bsize = 1024

    def __init__(self):
        self.DATABASES = copy.copy(settings.DATABASES)
        self.MEDIA_ROOT = copy.copy(settings.MEDIA_ROOT)
        self.PROJECT_PANEL = copy.copy(settings.PROJECT_PANEL)

    @classmethod
    def bytesto(cls, bytes, to):
        return bytes / (cls.bsize ** cls.UNIT_SIZE[to])

    @classmethod
    def bytes_display(cls, bytes):
        for i in reversed(list(cls.UNIT_SIZE.keys())):
            size = cls.bytesto(bytes, i)
            if size > 1:
                return (round(size, 2), i)
        return (round(cls.bytesto(bytes, to=cls.KB), 2), cls.KB)

    @classmethod
    def folders_files_size(cls, dir):
        size = 0
        files = 0
        for f in os.listdir(dir):
            path = os.path.join(dir, f)
            if os.path.isfile(path):
                size += os.path.getsize(path)
                files += 1
            else:
                res = cls.folders_files_size(path)
                size += res[0]
                files += res[1]
        return (size, files)

    def virtual_memory(self):
        ram = psutil.virtual_memory()
        return {
            'total': self.bytes_display(ram.total),
            'available': self.bytes_display(ram.available),
            'percent': ram.percent,
        }

    def hard_disk(self):
        hard = psutil.disk_usage('/')
        return {
            'total': self.bytes_display(hard.total),
            'free': self.bytes_display(hard.free),
            'percent': hard.percent,
        }

    def get_db_report(self, alias, table=None):
        return [{'name': i[0], 'size': self.bytes_display(i[1]), 'rows': i[2]} for i in DataBase(alias).report(table)]

    def databases(self):
        clean = self.PROJECT_PANEL.get('clean_model', {})
        result = []
        for alias, data in self.DATABASES.items():
            db = {'name': data['NAME'], 'alias': alias, 'tables': self.get_db_report(alias)}
            for table in db['tables']:
                table['clean'] = clean.get(f'{alias}.{table["name"]}')
            result.append(db)
        return result

    def folders_files(self):
        folders_files = copy.copy(self.PROJECT_PANEL.get('folders_files', []))
        folders_files.insert(0, {'path': self.MEDIA_ROOT, 'label': 'Медиа файлы'})
        for i in folders_files:
            size, count = self.folders_files_size(i['path'])
            folders_files[folders_files.index(i)] = {**i, 'size': self.bytes_display(size), 'count': count}
        return folders_files

    def clean_table(self, alias, *args, **kwargs):
        return DataBase(alias).clean_table(*args, **kwargs)

    def models(self, ):
        return [{'model': m, 'table': m._meta.db_table, 'rows': m.objects.all().count()} for c in apps.get_app_configs() for m in c.get_models()]


class DataBase:
    def __init__(self, alias):
        self.alias = alias
        self.db = copy.copy(settings.DATABASES)[self.alias]
        self.connection = connections[self.alias]

    def report(self, table=None):
        name = self.db['NAME']
        table = f" AND table_name = '{table}'" if table else ''

        sql_set = {
            'django.db.backends.mysql': "SELECT table_name AS `Table`, round((data_length + index_length), 2) AS `Size`, table_rows AS `Rows`"
                                        "FROM information_schema.TABLES "
                                        f"WHERE table_schema = '{name}'{table} "
                                        "ORDER BY round((data_length + index_length), 2) DESC;",
            'django.db.backends.postgresql': "select TABLE_NAME AS Table, pg_relation_size(quote_ident(TABLE_NAME)) AS SIZE, '-' AS Rows "
                                             "from information_schema.tables "
                                             f"where table_schema = 'public'{table} "
                                             "order BY 2 desc;"}
        sql = sql_set.get(self.db['ENGINE'])
        if not sql:
            raise Exception(f'Движок базы не поддерживается {self.db["ENGINE"]}')
        with self.connection.cursor() as c:
            c.execute(sql)
            return c.fetchall()

    def clean_table(self, table, field, value, bild_sql=False):
        sql_set = {
            'django.db.backends.mysql': f"DELETE FROM `{table}` WHERE `{field}` < '{value}'",
            'django.db.backends.postgresql': f"DELETE FROM `{table}` WHERE `{field}` < '{value}'"
        }
        sql = sql_set.get(self.db['ENGINE'])
        if not sql:
            raise Exception(f'Движок базы не поддерживается {self.db["ENGINE"]}')
        if bild_sql:
            return sql
        with self.connection.cursor() as c:
            c.execute(sql)





import copy
from datetime import datetime, timedelta, date

from django.utils import timezone
from django.contrib.auth.mixins import PermissionRequiredMixin, LoginRequiredMixin
from django.views import generic

from .models import *
from .project_panel import ProjectPanel


class Panel(LoginRequiredMixin, PermissionRequiredMixin, generic.TemplateView):
    permission_required = ['project_panel_app.view_panel']
    template_name = 'django_project_panel/panel.html'
    extra_context = {
        'title': 'Мониторинг-панель проекта',
    }

    def get_context_data(self, **kwargs):
        panel = ProjectPanel()
        ctx = super(Panel, self).get_context_data(**kwargs)
        ctx['virtual_memory'] = panel.virtual_memory()
        ctx['hard_memory'] = panel.hard_disk()
        ctx['folders_files'] = panel.folders_files()
        ctx['db'] = panel.databases()
        return ctx


class CleanTable(LoginRequiredMixin, PermissionRequiredMixin, generic.TemplateView):
    permission_required = ['project_panel_app.edit_panel']
    template_name = 'django_project_panel/clean_table.html'
    extra_context = {
        'title': 'Мониторинг-панель проекта',
    }
    result = ''

    def dispatch(self, request, *args, **kwargs):
        self.alias = kwargs['alias']
        self.table = kwargs['table']
        self.db = copy.copy(settings.DATABASES[self.alias])
        self.clean_settings = copy.copy(settings.PROJECT_PANEL.get('clean_model', {}).get(f'{self.alias}.{self.table}'))
        if self.clean_settings is None:
            raise Exception(f'В блоке clean_model нет сведений о таблице {self.alias}.{self.table} (settings.py)')
        self.panel = ProjectPanel()
        self.table_info = self.panel.get_db_report(self.alias, self.table)[0]
        self.filter_value = (timezone.now() - timedelta(days=30)).strftime('%Y-%m-%d')
        return super(CleanTable, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.filter_value = self.request.POST.get('filter_value', self.filter_value)
        if self.request.POST.get('run_sql_query') == 'run':
            try:
                self.panel.clean_table(self.alias, self.table, self.clean_settings['filter_field_lt'], self.filter_value)
                self.result = 'Запрос выполнен'
                self.table_info = self.panel.get_db_report(self.alias, self.table)[0]
            except Exception as e:
                self.result = f'Запрос не выполнен: {e}'
        return super(CleanTable, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(CleanTable, self).get_context_data(**kwargs)
        ctx['title'] = f'{self.alias}.{self.table}'
        ctx['alias'] = self.alias
        ctx['db'] = self.db['NAME']
        ctx['table'] = self.table
        ctx['table_info'] = self.table_info
        ctx['sql'] = self.panel.clean_table(self.alias, self.table, self.clean_settings['filter_field_lt'], self.filter_value, bild_sql=True)
        ctx['filter_value'] = self.filter_value
        ctx['result'] = self.result
        return ctx






import setuptools

# py setup.py sdist bdist_wheel
# py -m twine upload --repository pypi dist/*

with open("readme.txt", "r", encoding='utf-8') as fh:
	long_description = fh.read()

requirements = [
	"psutil<=5.9.5"
]

setuptools.setup(
	name="django-project-panel",
	version="0.0.5",
	author="EfrosiniaPetrovna",
	author_email="je.to.prace@gmail.com",
	description="Простая мониторинг-панель проекта (размеры всех таблиц в базе данных, медиа файлов и т.д)",
	long_description=long_description,
	long_description_content_type="text",
	url="https://github.com/EfrosiniaPetrovna/django-project-panel",
	packages=setuptools.find_packages(),
	install_requires=requirements,
	# Предоставляет pip некоторые метаданные о пакете. Также отображается на странице PyPi.
	classifiers=[
		"Programming Language :: Python :: 3.10",
	],
	# Требуемая версия Python.
	python_requires='>=3.6',
)


from django.urls import path, include

from .views import *


urlpatterns = [
    path('panel/', Panel.as_view(), name='project_panel'),
    path('clean_table/<alias>/<table>/', CleanTable.as_view(), name='project_panel_clean_table'),
]
